package cnfchecker;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.HashSet;

/**
 * Tests if a given instance file complies with the rules given here:
 * @see http://www.satcompetition.org/2011/format-benchmarks2011.html
 * @author S. Gerber
 */
public class CNFChecker {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        String line;
        String result = "";
        String warning = "";
        int numc = 0, numv = 0;
        
        int numc_read = 0;
        boolean error = false;
        boolean headerRead = false;
        int lc = 0;
        while ((line = br.readLine()) != null) {
            lc++;
            if ("".equals(line)) {
                break;
            }
            if (line.startsWith("c") && !headerRead) {
                continue;
            } else if (line.startsWith("c")) {
                result = "invalid: comments after header at line " + lc;
                error = true;
                break;
            } else if (line.startsWith("p") && headerRead) {
                result = "invalid: p line, but header already read at line " + lc;
                error = true;
                break;
            } else if (line.startsWith("p")) {
                String[] data = line.trim().split(" +");
                if (data.length != 4 || !"p".equals(data[0]) || !"cnf".equals(data[1])) {
                    result = "invalid header at line " + lc;
                    error = true;
                    break;
                }
                try {
                    numv = Integer.valueOf(data[2]);
                } catch (NumberFormatException e) {
                    result = "invalid header: could not parse #variables at line " + lc;
                    error = true;
                    break;
                }
                try {
                    numc = Integer.valueOf(data[3]);
                } catch (NumberFormatException e) {
                    result = "invalid header: could not parse #clauses at line " + lc;
                    error = true;
                    break;
                }
                headerRead = true;
            } else if (!headerRead) {
                result = "missing header at line " + lc;
                error = true;
                break;
            } else {
                numc_read++;
                String[] data = line.trim().split(" +");
                HashSet<Integer> litMap = new HashSet<Integer>();
                for (int i = 0; i < data.length-1; i++) {
                    try {
                        int lit = Integer.valueOf(data[i]);
                        if (lit == 0) {
                            result = "could not parse clause at line " + lc + " invalid literal: 0";
                            error = true;
                            break;
                        }
                        if (lit > numv || -lit > numv) {
                            result = "literal " + lit + " invalid at line " + lc;
                            error = true;
                            break;
                        }
                        if (litMap.contains(-lit)) {
                            warning = "clause contains " + lit + " and " + (-lit) + " at line " + lc;
                           // error = true;
                            break;
                        }
                        litMap.add(lit);
                    } catch (NumberFormatException e) {
                        result = "could not parse literal at line " + lc;
                        error = true;
                        break;
                    }
                }                
                if (error) {
                    break;
                }
                try {
                    int nullvalue = Integer.valueOf(data[data.length-1]);
                    if (nullvalue != 0) {
                        result = "unexpected end of clause at line " + lc;
                        error = true;
                        break;
                    }
                } catch (NumberFormatException ex) {
                    result = "could not parse last character at line " + lc;
                    error = true;
                    break;
                }
            }
        }
		if (!error && !headerRead) {
			error = true;
			result = "invalid: no header (p line)";
		}
        if (!error) {
            if (numc != numc_read) {
                result = "invalid number of clauses or empty line";
                error = true;
            }
        }
        
        if (error) {
            System.out.println(result);
        } else {
            System.out.println("valid" + ("".equals(warning) ? "" : " (warning: " + warning + ")"));
        }
    }
}
